

import sys
import shutil
import py.test

from tiddlyweb.store import Store
from tiddlyweb.model.recipe import Recipe
from tiddlyweb.config import config
from tiddlyweb.manage import handle
from tiddlywebplugins.utils import get_store
from tiddlywebplugins.recipeupdater import init


def setup_module(module):
    try:
        shutil.rmtree('store')
    except:
        pass
    module.store = get_store(config)
    init(config)
    base_recipe = Recipe('hi')
    base_recipe.set_recipe([
        ('system', ''),
        ('stuff', 'select=title:monkey'),
        ('things', '')])
    module.store.put(base_recipe)
    sys.exit = boring_exit


class InternalExit(Exception):
    pass


def boring_exit(value):
    raise InternalExit()


def test_simple_replace():
    handle(['', 'recipeupdate', '/bags/stuff/tiddlers?select=title:monkey',
        '/bags/faster/tiddlers'])

    recipe = store.get(Recipe('hi'))
    recipelist = recipe.get_recipe()

    assert ['faster', ''] in recipelist
    assert ['stuff', 'select=title:monkey'] not in recipelist


def test_double_replace():
    handle(['', 'recipeupdate', '/bags/faster/tiddlers',
        '/bags/thanks/tiddlers?sort=modified',
        '/bags/collapse/tiddlers'])

    recipe = store.get(Recipe('hi'))
    recipelist = recipe.get_recipe()

    assert ['faster', ''] not in recipelist
    assert ['thanks', 'sort=modified'] in recipelist
    assert ['collapse', ''] in recipelist


def test_bad_input():
    py.test.raises(InternalExit, "handle(['', 'recipeupdate', '/bags/collapse/tiddlers', '/bags/thanks/tiddlers?sort=modified', '/bags/tiddlers'])")
    py.test.raises(InternalExit, "handle(['', 'recipeupdate', '/bags/tiddlers', '/bags/thanks/tiddlers?sort=modified', '/bags/collapse/tiddlers'])")




# YOU NEED TO EDIT THESE
AUTHOR = 'Chris Dent'
AUTHOR_EMAIL = 'cdent@peermore.com'
NAME = 'tiddlywebplugins.recipeupdater'
DESCRIPTION = 'Update a recipe entry with one or more other entries'
VERSION = '0.1'


import os

from setuptools import setup, find_packages


# You should carefully review the below (install_requires especially).
setup(
    namespace_packages = ['tiddlywebplugins'],
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = open(os.path.join(os.path.dirname(__file__), 'README')).read(),
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = 'http://pypi.python.org/pypi/%s' % NAME,
    platforms = 'Posix; MacOS X; Windows',
    packages = find_packages(exclude=['test']),
    install_requires = ['setuptools',
        'tiddlyweb',
        'tiddlywebplugins.utils'],
    zip_safe = False
    )



"""
A twanager command that updates all the recipes in the store
that match a bag filter combo with one or more replacement
bag filter combos:

twanager recipeupdate '/bags/foobar/tiddlers?select=bang' \
    '/bags/foobar/tiddlers' '/bags/anotherbag/tiddlers?select=bag'

To use, add 'tiddlywebplugins.recipeupdater' to twanager_plugins
in tiddlywebconfig.py
"""


from tiddlyweb.manage import make_command
from tiddlywebplugins.utils import get_store

def init(config):

    @make_command()
    def recipeupdate(args):
        """Update all recipes entry with a different entry. <match bag info> <replacement bag info> [<replacement bag info>]"""
        store = get_store(config)
        match_bag = args.pop(0)
        bag, filter = _text_to_tuple(match_bag)
        replace_bags = args
        for recipe in store.list_recipes():
            recipe = store.get(recipe)
            recipe_list = recipe.get_recipe()
            try:
                index = recipe_list.index([bag, filter])
            except ValueError:
                # not in the recipe carry on
                continue
            recipe_list.pop(index)
            for bag_string in reversed(replace_bags):
                bag, filter = _text_to_tuple(bag_string)
                recipe_list.insert(index, (bag, filter))
            recipe.set_recipe(recipe_list)
            store.put(recipe)


def _text_to_tuple(bag_string):
    try:
        bag_string, filter_string = bag_string.split('?')
    except ValueError:
        bag_string = bag_string
        filter_string = ''
    if not filter_string:
        filter_string = ''
    bag_bits = bag_string.split('/')
    if len(bag_bits) < 4:
        raise ValueError('bag string malformed %s' % bag_string)
    bag = bag_bits[2]
    return bag, filter_string

